#include "mex.h"

#include "stdlib.h"
#include "eGateHighSpeedPort.h"
#include "windows.h"
#include "time.h"


#define INVALID_WORKMODE            (0)
#define WORKMODE_CONNECT            (1)
#define WORKMODE_SETBUFFER          (2)
#define WORKMODE_GETNUMBEROFCHANNEL (3)
#define WORKMODE_GETCHANNELNAME		(4)
#define WORKMODE_READDATA           (5)
#define WORKMODE_CLOSE_CONNECTION   (6)
#define WORKMODE_GETSAMPLERATE      (7)
#define WORKMODE_SLEEP              (8)


typedef int (__cdecl* MY_DLL_TYPE_INIT)(char*,int,int,int,int*,int*);
typedef int (__cdecl* MY_DLL_TYPE_INITBUFFER)(int,int,int);
typedef int (__cdecl* MY_DLL_TYPE_GETNUMBEROFCHANNELS)(int,int,int*);
typedef int (__cdecl* MY_DLL_TYPE_GETCHANNELINFOSTRING)(int,int,int,int,char*);
typedef int (__cdecl* MY_DLL_TYPE_STARTCONFIGURATION)();
typedef int (__cdecl* MY_DLL_TYPE_READBUFFERNEXTCHANNEL)(int,int);
typedef int (__cdecl* MY_DLL_TYPE_READBUFFERSINGLE)(int,int,int,double*);
typedef int (__cdecl* MY_DLL_TYPE_CLOSE)(int,int);
typedef int (__cdecl* MY_DLL_TYPE_GETSAMPLERATE)(int,int,int,double*,char);
typedef int (__cdecl* MY_DLL_TYPE_SLEEP)(int);



void PrintUsage(){
    printf("This is the MexFunction read_exact.\n");
    printf("With the help of this MexFunction data can be read from an e.xact system.\n");
    printf("Usage:\n");
    printf("read_exact(workmode,...)\n");
    printf("workmode:\n");
    printf("\t1 - connect to hardware with specified ip address\n");
    printf("\t2 - set the buffer index to specify which connection should be used\n");
    printf("\t3 - get number of channels of the selected buffer\n");
    printf("\t4 - get channel name\n");    
    printf("\t5 - read data\n");    
	printf("\t6 - close connection\n"); 
    printf("\t7 - get samplerate\n"); 
    printf("\t8 - sleep (ms)\n"); 
    
	printf("\n");
	printf("Workmode 1: Connect to hardware\n");
	printf("Additional parameters:\n");
	printf("\tIP address of the hardware to connect\n");
	printf("\tdata source\n");
	printf("Return:\n");
	printf("\tError code\n");
    printf("Example: ret = read_exact(1,\"192.141.151.12\",2)\n");

	printf("\n");
	printf("Workmode 2: Set buffer index\n");
	printf("Additional parameters:\n");
	printf("\tBuffer index\n");
	printf("Return:\n");
	printf("\tError code\n");
    printf("Example: ret = read_exact(2,0)\n");

	printf("\n");
	printf("Workmode 3: Get number of channels\n");
	printf("Additional parameters:\n");
	printf("\tnone\n");
	printf("Return:\n");
	printf("\tError code\n");
	printf("\tNumber of channels\n");
    printf("Example: [ret,NumberOfChannels] = read_exact(3)\n");

	printf("\n");
	printf("Workmode 4: Get channel name\n");
	printf("Additional parameters:\n");
	printf("\tChannel index (zero based)\n");
	printf("Return:\n");
	printf("\tError code\n");
	printf("\tChannel name\n");
    printf("Example: [ret,ChannelName] = read_exact(4,0)\n");

	printf("\n");
	printf("Workmode 5: Read data\n");
	printf("Additional parameters:\n");
	printf("\tFirst set of data to read (0..no, 1..yes ->1 will force a re-init of the reading process)\n");
	printf("\tNumber of channels\n");
	printf("\tRow vector of channel indices to read\n");
	printf("Return:\n");
	printf("\tError code\n");
	printf("\tRow vector of read values\n");
    printf("Example: [ret,values] = read_exact(5,1,14,[0,2,5])\n");

	printf("\n");
	printf("Workmode 6: Close connection\n");
	printf("Additional parameters:\n");
	printf("Return:\n");
	printf("\tError code\n");
    printf("Example: [ret] = read_exact(6)\n");



}

void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{

    static int HCLIENT = -1;		//Sometimes different routines from one application need to access the same values
    static int HCONNECTION = -1;	//Up to 10 connections to the same or to different controllers can be initialized
									//Both identifiers shoule be initialized with -1 first!
									
	int ret=0;						//Return coede for the HighSpeedPort functions
	int time_ms;                    //sleep time in ms
    int channelIndex=-1;
	int channelCount=0;
    char channelInfo;
	double *rChannelCount;
    double *rSampleRate;
	int i=0;
	int workmode = INVALID_WORKMODE;
	int buffermode = HSP_BUFFER;

	double value=0;					

	int index= 0;
	char info[1024]="";
	char *ip;
    int   buflen,status;
    int dims[2];
	double *rRet;
	
	static HINSTANCE		HandleLIB = NULL;
	static MY_DLL_TYPE_INIT dll_init = NULL;
	static MY_DLL_TYPE_INITBUFFER dll_init_buffer = NULL;
	static MY_DLL_TYPE_GETNUMBEROFCHANNELS dll_get_number_of_channels = NULL;
	static MY_DLL_TYPE_GETCHANNELINFOSTRING dll_get_channel_info_string = NULL;
	static MY_DLL_TYPE_STARTCONFIGURATION dll_start_configuration = NULL;
	static MY_DLL_TYPE_READBUFFERNEXTCHANNEL dll_read_buffer_next_channel = NULL;
	static MY_DLL_TYPE_READBUFFERSINGLE dll_read_buffer_single = NULL;
	static MY_DLL_TYPE_CLOSE dll_close = NULL;	
    static MY_DLL_TYPE_GETSAMPLERATE dll_get_samplerate = NULL;
    static MY_DLL_TYPE_SLEEP dll_sleep = NULL;
	
	if(HandleLIB == NULL){
	    //war noch nicht geladen->jetzt machen
    	HandleLIB = LoadLibrary("eGateHighSpeedPort.dll");
    }
	if (HandleLIB){
	    if(dll_init == NULL){
		    dll_init = (MY_DLL_TYPE_INIT)GetProcAddress(HandleLIB,
									"_CD_eGateHighSpeedPort_Init");
		}
	    if(dll_init_buffer == NULL){		
		dll_init_buffer = (MY_DLL_TYPE_INITBUFFER)GetProcAddress(HandleLIB,
									"_CD_eGateHighSpeedPort_InitBuffer");
        }									
	    if(dll_get_number_of_channels == NULL){									
		dll_get_number_of_channels = (MY_DLL_TYPE_GETNUMBEROFCHANNELS)GetProcAddress(HandleLIB,
									"_CD_eGateHighSpeedPort_GetNumberOfChannels");
		}
	    if(dll_get_channel_info_string == NULL){									
		dll_get_channel_info_string = (MY_DLL_TYPE_GETCHANNELINFOSTRING)GetProcAddress(HandleLIB,
									"_CD_eGateHighSpeedPort_GetChannelInfo_String");
		}
	    if(dll_start_configuration == NULL){									
		dll_start_configuration = (MY_DLL_TYPE_STARTCONFIGURATION)GetProcAddress(HandleLIB,
									"_CD_eGateHighSpeedPort_StartConfiguration");
		}
	    if(dll_read_buffer_next_channel == NULL){									
		dll_read_buffer_next_channel = (MY_DLL_TYPE_READBUFFERNEXTCHANNEL)GetProcAddress(HandleLIB,
									"_CD_eGateHighSpeedPort_ReadBuffer_NextFrame");
		}
	    if(dll_read_buffer_single == NULL){									
		dll_read_buffer_single = (MY_DLL_TYPE_READBUFFERSINGLE)GetProcAddress(HandleLIB,
									"_CD_eGateHighSpeedPort_ReadBuffer_Single");
		}
	    if(dll_close == NULL){									
		dll_close = (MY_DLL_TYPE_CLOSE)GetProcAddress(HandleLIB,
									"_CD_eGateHighSpeedPort_Close");
        }
        if(dll_get_samplerate == NULL){									
		dll_get_samplerate = (MY_DLL_TYPE_GETSAMPLERATE)GetProcAddress(HandleLIB,
									"_CD_eGateHighSpeedPort_GetDeviceInfo");
        }
        if(dll_sleep == NULL){									
		dll_sleep = (MY_DLL_TYPE_SLEEP)GetProcAddress(HandleLIB,
									"_CD_eGateHighSpeedPort_SleepMS");
        }
        
	}
	else{
		dll_init = NULL;
		dll_init_buffer = NULL;
		dll_get_number_of_channels = NULL;
		dll_get_channel_info_string = NULL;
		dll_start_configuration = NULL;
		dll_read_buffer_next_channel = NULL;
		dll_read_buffer_single = NULL;
		dll_close = NULL;
        dll_get_samplerate = NULL;
        dll_sleep = NULL;
	}
    if((dll_init == NULL)||(dll_init_buffer == NULL)||(dll_get_number_of_channels == NULL)||(dll_get_channel_info_string == NULL)||
    (dll_read_buffer_next_channel == NULL)||(dll_read_buffer_next_channel == NULL)||(dll_read_buffer_single == NULL)||(dll_close == NULL)||
    (dll_get_samplerate == NULL)||(dll_sleep == NULL)){
        //Dll nicht korrekt geladen
        mexErrMsgTxt("Dll not loaded correctly. Terminate.\n");
    	dll_close(HCONNECTION, HCLIENT);
    	return;
    }
	
    //Dll is now loaded -> check parameters, depending on work mode
    //Work modes:
    //1 - connect to hardware
    //2 - set buffer index
    //3 - get number of channels of the selected buffer and get channel info
    //4 - start configuration
    //5 - read data
    //7 - get sample rate
    //8 - sleep
    
    if(nrhs < 1){
        //error, no work mode specified
        PrintUsage();
        mexErrMsgTxt("No work mode specified\n");
        return;
    }

    if(mxIsChar(prhs[0]) || mxIsComplex(prhs[0]) || !(mxGetM(prhs[0]) == 1 && mxGetN(prhs[0]) == 1) ) {
        mexErrMsgTxt("Parameter 1 must be a noncomplex scalar integer.");
    }
    else{
        workmode = (int)*mxGetPr(prhs[0]);
    }
    //+++Connect
    if(workmode == WORKMODE_CONNECT){
        //IP holen
        if(nrhs < 2){
            PrintUsage();
            mexErrMsgTxt("No ip address specified\n");
            return;
        }
        if(mxIsChar(prhs[1]) != 1){
            PrintUsage();
            mexErrMsgTxt("Parameter 2 must be a string.");
            return;
        }
        if(mxGetM(prhs[1]) != 1){
            mexErrMsgTxt("Parameter 2 must be a row vector.");
        }

        buflen = (mxGetM(prhs[1]) * mxGetN(prhs[1])) + 1;
        ip = mxCalloc(buflen, sizeof(char));
        status = mxGetString(prhs[1], ip, buflen);
        if(status != 0){
            mexWarnMsgTxt("Not enough space. String is truncated.");
        }
        //get transfer mode (HSP_BUFFER or HSP_LOGGER)
        if(nrhs < 3){
            PrintUsage();        
            mexErrMsgTxt("No transfer mode specified\n");
            return;
        }
        if(mxIsChar(prhs[2]) || mxIsComplex(prhs[2]) || !(mxGetM(prhs[2]) == 1 && mxGetN(prhs[2]) == 1) ) {
            mexErrMsgTxt("Paremeter 3 must be a noncomplex scalar integer.");
        }
        else{
            buffermode = (int)*mxGetPr(prhs[2]);
        }

		if(nlhs != 1){
            PrintUsage();
            mexErrMsgTxt("Not enough output arguments specified\n");
            return;
        }
        
        
        ret = dll_init(                   ip,				//IP address of the controller as string
				    					  5,				//Connection timeout in seconds as integer
								    	  buffermode,		//Transfer mode as integer (see header file for constants)
    									  5,				//Sample reate: defines the interval for reading data between device and PC
				    					  &HCLIENT,			//The function returns the ID's for the client and the connection
								    	  &HCONNECTION);

		plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
		rRet = mxGetPr(plhs[0]);
	    rRet[0] = (double)ret;
								    	  
    	if(ret!=HSP_OK){
        	printf("Initialization not successful. Terminate.\n");
			//Hier kein close, da noch gar nichts offen war, bei aufruf crasht dll
        	//dll_close(HCONNECTION, HCLIENT);
        	return;
    	}
    	return;
    }
    //---Connect
									  
    //+++SetBuffer
    if(workmode == WORKMODE_SETBUFFER){
        //Bufferindex holen
        if(nrhs < 2){
            PrintUsage();
            mexErrMsgTxt("No buffer index specified\n");
            return;
        }
        if(mxIsChar(prhs[1]) || mxIsComplex(prhs[1]) || !(mxGetM(prhs[1]) == 1 && mxGetN(prhs[1]) == 1) ) {
          mexErrMsgTxt("Paremeter 2 must be a noncomplex scalar integer.");
        }
        else{
            index = (int)*mxGetPr(prhs[1]);
        }

		if(nlhs != 1){
            PrintUsage();
            mexErrMsgTxt("Not enough output arguments specified\n");
            return;
        }

        ret = dll_init_buffer(HCONNECTION,  //Tells the DLL which connection sould be used
							index,          //The buffer index on the controller (can be up to 10)
							1);			    //AutoMode

		plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
		rRet = mxGetPr(plhs[0]);
	    rRet[0] = (double)ret;

    	if(ret!=HSP_OK){
             printf("Buffer initialization not successful. Ret %d. Terminate.\n",ret);
         	 dll_close(HCONNECTION, HCLIENT);
        	 return;
    	}   
    	return;
    }
    //---SetBuffer
	
    //+++GetNumberOfChannel
    if(workmode == WORKMODE_GETNUMBEROFCHANNEL){
		int channelCount=0;
        if(nlhs != 2){
            PrintUsage();
            mexErrMsgTxt("Not enough output arguments specified\n");
            return;
        }
    	//Read number of channels of the selected buffer
    	ret = dll_get_number_of_channels(HCONNECTION,		//-"-
										 DADI_INPUT,		//Only input channels
										 &channelCount);	//Number of channels

		plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
		rRet = mxGetPr(plhs[0]);
	    rRet[0] = (double)ret;

    	if(ret!=HSP_OK){
	        printf("Number of channels not retrieved. Ret %d. Terminate.\n",ret);
        	dll_close(HCONNECTION, HCLIENT);
        	return;
	    }
	
		plhs[1] = mxCreateDoubleMatrix(1,1, mxREAL);
		rChannelCount = mxGetPr(plhs[1]);
	    rChannelCount[0] = (double)channelCount;

		return;
	}
	
	    //+++GetChannelInfo
    if(workmode == WORKMODE_GETCHANNELNAME){
		char *pMyStrings = NULL;
		int channelindex=0;
        if(nlhs != 2){
            PrintUsage();
            mexErrMsgTxt("Not enough output arguments specified\n");
            return;
        }

		if(mxIsChar(prhs[1]) || mxIsComplex(prhs[1]) || !(mxGetM(prhs[1]) == 1 && mxGetN(prhs[1]) == 1) ) {
            mexErrMsgTxt("Paremeter 2 must be a noncomplex scalar integer.");
        }
        else{
            channelindex = (int)*mxGetPr(prhs[1]);
        }
	
		//Read the channel name
		ret =	dll_get_channel_info_string(HCONNECTION,
											 CHINFO_NAME,	//Type of info = name
											 DADI_INPUT,	//Input channels			
											 channelindex,				//channel index
											 info);			//info

		plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
		rRet = mxGetPr(plhs[0]);
		rRet[0] = (double)ret;
				
		buflen = strlen(info);
		pMyStrings = mxCalloc(buflen, sizeof(char));
		plhs[1] = mxCreateString(info);
    	if(ret!=HSP_OK){
			printf("Channel info not retrieved. Terminate.\n");
        	dll_close(HCONNECTION, HCLIENT);
        	return;
		}
		return;
	}

	if(workmode == WORKMODE_READDATA){
		double *channels_indices;
		int iNumChannels = 0;
		int channelCount=0;
		double *outputs;
		int restart;
		//---GetChannelInfo	
		if(nrhs != 4){
			PrintUsage();
            mexErrMsgTxt("Not enough input arguments specified\n");
            return;
        }
		if(mxGetM(prhs[3]) != 1){
			PrintUsage();
            mexErrMsgTxt("Parameter 4 must be a row vector.\n");
		}
		
		if(mxIsChar(prhs[1]) || mxIsComplex(prhs[1]) || !(mxGetM(prhs[1]) == 1 && mxGetN(prhs[1]) == 1) ) {
			mexErrMsgTxt("Parameter 2 must be a noncomplex scalar integer.");
		}
        else{
			restart = (int)*mxGetPr(prhs[1]);
        }

		if(mxIsChar(prhs[2]) || mxIsComplex(prhs[2]) || !(mxGetM(prhs[2]) == 1 && mxGetN(prhs[2]) == 1) ) {
			mexErrMsgTxt("Parameter 3 must be a noncomplex scalar integer.");
		}
        else{
			channelCount = (int)*mxGetPr(prhs[2]);
        }
		if(nlhs != 2){
			PrintUsage();
            mexErrMsgTxt("Not enough output arguments specified\n");
            return;
		}
		
		
		channels_indices = mxGetPr(prhs[3]);
		iNumChannels = mxGetN(prhs[3]);

		plhs[1] = mxCreateDoubleMatrix(1,iNumChannels, mxREAL);
		outputs = mxGetPr(plhs[1]);
		
		for(i=0;i<iNumChannels;++i){
			if((channels_indices[0]<0 || channels_indices[0]>=channelCount)){
	    		printf("Channel out of range. Terminate.\n");
				dll_close(HCONNECTION, HCLIENT);
    			return;
			}
		}
		if(restart == 1){
			dll_start_configuration();		//Start the internal threads
		}
	 
        
/*
         printf("read data \n");
*/
        
		ret = HSP_NOT_READY;
		while( ret == HSP_NOT_READY)
        {
			ret = dll_read_buffer_next_channel(HCONNECTION,HCLIENT);   //Step to the next frame


			if (ret == HSP_NOT_READY)
            {
            //printf("ret = %d \n",ret);
            dll_sleep(100);
            }   
            
		}
		if(ret==HSP_OK){
           
			for(i=0;i<iNumChannels;++i){
				ret = dll_read_buffer_single(HCONNECTION,HCLIENT,channels_indices[i],&value);  //Read values by channelIndex
				//This function would read the same value until "NextChannel" was called and returned HSP_OK
            
                outputs[i] = value;
			}
		}
		else if(ret == HSP_NOT_READY){
			//Not ready, dll needs some time to read and decode data
            Sleep(1);
			printf("Not ready");
			plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
			rRet = mxGetPr(plhs[0]);
			rRet[0] = (double)ret;
		}
		else{
			plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
			rRet = mxGetPr(plhs[0]);
			rRet[0] = (double)ret;
			printf("Error occured while reading online values");
		}
		plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
		rRet = mxGetPr(plhs[0]);
		rRet[0] = (double)ret;

		return;
	}
	if(workmode == WORKMODE_CLOSE_CONNECTION){
		if(nlhs != 1){
			PrintUsage();
            mexErrMsgTxt("Not enough output arguments specified\n");
            return;
		}

		if((HCLIENT != -1) && (HCONNECTION != -1)){
			dll_close(HCONNECTION, HCLIENT);
		}
		plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
		rRet = mxGetPr(plhs[0]);
		rRet[0] = (double)0;
	}
    
    
    
    //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
     if(workmode == WORKMODE_GETSAMPLERATE){
    
       if(nlhs != 2){
            PrintUsage();
            mexErrMsgTxt("Not enough output arguments specified\n");
            return;
        }
    	//Read samplerate
    	ret = dll_get_samplerate(HCONNECTION,		//-"-
										 16,		//typeID=16 for samplerate
										 1,         //index
                                         &value  ,
                                         channelInfo);        

		plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
		rRet = mxGetPr(plhs[0]);
	    rRet[0] = (double)ret;

    	if(ret!=HSP_OK){
	        printf("Sample Rate not retrieved. Ret %d. Terminate.\n",ret);
        	dll_close(HCONNECTION, HCLIENT);
        	return;
	    }
	
        plhs[1] = mxCreateDoubleMatrix(1,1, mxREAL);
		rSampleRate = mxGetPr(plhs[1]);
	    rSampleRate[0] = (double)value;

		return;
        
    }
     //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
    //%%%%%%%%SLEEP%%%%%%%%%%%%%%
    	if(workmode == WORKMODE_SLEEP){
        
            int time_ms = 0;
            
        
            if(nrhs != 2){
            PrintUsage();
            mexErrMsgTxt("Wrong number of input arguments. Use read_exact(8,time_ms)\n");
            return;
        }
        
        if(mxIsChar(prhs[1]) || mxIsComplex(prhs[1]) || !(mxGetM(prhs[1]) == 1 && mxGetN(prhs[1]) == 1) ) {
            mexErrMsgTxt("Paremeter 2 must be a noncomplex scalar integer.");
        }
        else{
            time_ms = (int)*mxGetPr(prhs[1]);
        }
            
            

        
    	ret = dll_sleep(time_ms
                );        

		plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
		rRet = mxGetPr(plhs[0]);
	    rRet[0] = (double)ret;

    	if(ret!=HSP_OK){
	        printf("error in Sleep (read_exact.c). Ret %d. Terminate.\n",ret);
        	dll_close(HCONNECTION, HCLIENT);
        	return;
	    }
        return;
    }
    //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    
  
    
    return;
}