#include "mex.h"

#include "stdlib.h"
#include "eGateUtility.h"
#include "windows.h"
#include "time.h"


#define INVALID_WORKMODE                 (0)
#define WORKMODE_IDENTIFY_FIRST          (1)
#define WORKMODE_IDENTIFY_NEXT           (2)

typedef int (__cdecl* MY_DLL_TYPE_IDENTIFY_FIRST)(long,double,char*,char*,char*);
typedef int (__cdecl* MY_DLL_TYPE_IDENTIFY_NEXT)(char*,char*,char*);

void PrintUsage(){
    printf("This is the MexFunction read_exact.\n");
}

void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
	long	I,
			retsub;
	char	h_Error[1024],
			h_DeviceInfo[1024],
			h_AdapterInfo[1024];
    
    
    int ret=0;
    int workmode = INVALID_WORKMODE;
    int   buflen,status;
    double *rRet;
    
    
    static HINSTANCE HandleLIB = NULL;
    static MY_DLL_TYPE_IDENTIFY_FIRST dll_identify_first = NULL;
    static MY_DLL_TYPE_IDENTIFY_NEXT dll_identify_next = NULL;
    
    if(HandleLIB == NULL){
        //war noch nicht geladen->jetzt machen
        HandleLIB = LoadLibrary("eGateUtility.dll");
    }    
    

    if (HandleLIB){
        if(dll_identify_first == NULL){
            dll_identify_first = (MY_DLL_TYPE_IDENTIFY_FIRST)GetProcAddress(HandleLIB,
                                    "_CDDLG_IdentifyDevices_First");
        }
        if(dll_identify_next == NULL){		
            dll_identify_next = (MY_DLL_TYPE_IDENTIFY_NEXT)GetProcAddress(HandleLIB,
                                    "_CDDLG_IdentifyDevices_Next");
        }   
    }
    else{
        dll_identify_first = NULL;
        dll_identify_next = NULL;
    }

    if((dll_identify_first == NULL)||(dll_identify_next == NULL)){
        //Dll nicht korrekt geladen
        mexErrMsgTxt("Dll not loaded correctly. Terminate.\n");
/*
    	dll_close(HCONNECTION, HCLIENT);
*/
    	return;
    }

    if(nrhs < 1){
        //error, no work mode specified
        PrintUsage();
        mexErrMsgTxt("No work mode specified\n");
        return;
    }
    
    if(mxIsChar(prhs[0]) || mxIsComplex(prhs[0]) || !(mxGetM(prhs[0]) == 1 && mxGetN(prhs[0]) == 1) ) {
        mexErrMsgTxt("Parameter 1 must be a noncomplex scalar integer.");
    }
    else{
        workmode = (int)*mxGetPr(prhs[0]);
    }    
    
    
    
    
    
    
    
    
   //+++Identify Device First
    if(workmode == WORKMODE_IDENTIFY_FIRST){
		char *pMyStrings = NULL;

        if(nlhs != 4){ //4 output arguments? AdapterInfo, DeviceInfo, Error
            PrintUsage();
            mexErrMsgTxt("Not enough output arguments specified\n");
            return;
        }

		//Identify Devices
		ret =	dll_identify_first(0, //UseExtCommand
											 1.0,	//ScanTime
											 h_AdapterInfo,		
											 h_DeviceInfo,
											 h_Error);

		plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
		rRet = mxGetPr(plhs[0]);
		rRet[0] = (double)ret;
				
		buflen = strlen(h_AdapterInfo);
		pMyStrings = mxCalloc(buflen, sizeof(char));
		plhs[1] = mxCreateString(h_AdapterInfo);
        
        buflen = strlen(h_DeviceInfo);
		pMyStrings = mxCalloc(buflen, sizeof(char));
		plhs[2] = mxCreateString(h_DeviceInfo);
        
        buflen = strlen(h_Error);
		pMyStrings = mxCalloc(buflen, sizeof(char));
		plhs[3] = mxCreateString(h_Error);
        
        
        
        
    	if(ret!=0){
			printf("Error in read_utility.c - identify device first. Terminate.\n");
        	return;
		}
		return;
	}
    
    
    
    
    
       //+++Identify Device Next
    if(workmode == WORKMODE_IDENTIFY_NEXT){
		char *pMyStrings = NULL;

        if(nlhs != 4){ //4 output arguments? AdapterInfo, DeviceInfo, Error
            PrintUsage();
            mexErrMsgTxt("Not enough output arguments specified\n");
            return;
        }

		//Identify Devices
		ret =	dll_identify_next(	 h_AdapterInfo,		
									 h_DeviceInfo,
									 h_Error);

		plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
		rRet = mxGetPr(plhs[0]);
		rRet[0] = (double)ret;
				
		buflen = strlen(h_AdapterInfo);
		pMyStrings = mxCalloc(buflen, sizeof(char));
		plhs[1] = mxCreateString(h_AdapterInfo);
        
        buflen = strlen(h_DeviceInfo);
		pMyStrings = mxCalloc(buflen, sizeof(char));
		plhs[2] = mxCreateString(h_DeviceInfo);
        
        buflen = strlen(h_Error);
		pMyStrings = mxCalloc(buflen, sizeof(char));
		plhs[3] = mxCreateString(h_Error);
        
        
        
        
    	if(ret!=0){
/*
			printf("Error in read_utility.c - identify device next. Terminate.\n");
*/
        	return;
		}
        
		return;
	}
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
        return;
}